import java.io.IOException;
import java.util.*;
import java.math.*;
import static java.lang.Math.*;

class Task {

	public static void moveMonster(CMonster m) {
		m.PositionX += m.VelocityX;
		m.PositionY += m.VelocityY;
	}

	public static void moveMissile(CMissile m) {
		m.PositionX += m.VelocityX;
		m.PositionY += m.VelocityY;
	}

	public static float d2(float ax, float ay, float bx, float by) {
		return (ax - bx) * (ax - bx) + (ay - by) * (ay - by);

	}

	public static float f(float x, float y, CPlayer player,
			CMonster[] monsters, CMissile[] misslies, World w, float cx,
			float cy) {
		float r = 0.0f;
		float pr = player.Radius * player.Radius;

		// damage
		float d = 0.0f;
		// distance
		float c = 0.0f;
		float q2 = (float) Math.sqrt(2);

		float pd = (float) Math.sqrt(d2(x, y, player.PositionX,
				player.PositionY)) / q2 + 1;

		float dc = min(min(d2(x, y, 0, 0), d2(x, y, 0, 1)),
				min(d2(x, y, 1, 0), d2(x, y, 1, 1)))
				/ q2 + 1;

		float dcm = (float) Math.sqrt(d2(x, y, cx, cy)) + 1;

		for (int i = 0; i < misslies.length; i++) {
			if (misslies[i].ClassName < 0) {
				continue;
			}
			if (d2(x, y, misslies[i].PositionX, misslies[i].PositionY) <= pr) {
				d += misslies[i].Damage;
			}
		}

		for (int i = 0; i < monsters.length; i++) {
			if (monsters[i].ClassName < 0) {
				continue;
			}

			float ctom = (float) Math.sqrt(d2(x, y, monsters[i].PositionX,
					monsters[i].PositionY));

			if (ctom <= player.Radius + monsters[i].Radius) {
				d += monsters[i].BiteDamage;
			}
			c += (1 - ctom / q2);

		}

		if (d >= player.HitPoints) {
			return Float.MAX_VALUE;
		}

		float dp = d / player.HitPoints;

		// System.out.printf("%.2f %.2f\n", c, dp);

		r = (c + dp * 1000 + dc) * pd * pd;
		r /= (dcm * 1.05);
		return r;
	}

	public static void main(String[] args) throws IOException {
		if (args.length < 2) {
			System.err.println("parameters: IP port");
			System.exit(10);
		}
		World world = new World(args[0], Integer.parseInt(args[1]));
		while (true) {
			world.makeMove();

			// System.out.println("frame : " + world.player.Time + " HP: "
			// + world.player.HitPoints + " x: " + world.player.PositionX
			// + " y: " + world.player.PositionY);

			float mind = Float.MAX_VALUE;
			float CX = 0.0f, CY = 0.0f;
			int count = 0;
			for (int i = 0; i < world.MAXMONSTERS; i++) {
				if (world.monsters[i].ClassName < 0)
					continue;

				CMonster c = world.monsters[i];
				float td = d2(world.player.PositionX, world.player.PositionY,
						c.PositionX, c.PositionY);
				if (td < mind) {
					mind = td;
					world.shoot(c.PositionX - world.player.PositionX,
							c.PositionY - world.player.PositionY);

				}
				moveMonster(world.monsters[i]);
				CX += c.PositionX;
				CY += c.PositionY;
				count++;

			}
			CX /= count;
			CY /= count;

			for (int i = 0; i < world.MAXMISSILES; i++) {
				if (world.missiles[i].ClassName < 0)
					continue;
				moveMissile(world.missiles[i]);
			}

			// kill monsters

			int n = 15;

			float min = 10000f;
			float minx = 0.0f;
			float miny = 0.0f;

			float r = 0.5f;

			for (int i = 0; i < n; i++) {
				for (int j = 0; j < n; j++) {
					float ty = (1.0f / n) * i;
					float tx = (1.0f / n) * j;

					if (d2(tx, ty, 0.5f, 0.5f) >= r * r) {
						continue;
					}

					double dp = sqrt(d2(world.player.PositionX,
							world.player.PositionY, tx, ty));
					if (dp > 0.65f) {
						continue;
					}

					float q = f(tx, ty, world.player, world.monsters,
							world.missiles, world, CX, CY);
					// System.out.printf("%.2f %.2f %.2f\n", tx, ty, q);

					if (q < min) {
						// System.out.println(q);
						min = q;
						minx = tx;
						miny = ty;
					}

				}
			}
			// System.out.printf("%.2f %.2f %.2f\n", minx, miny, min);
			// System.in.read();
			world.run(minx - world.player.PositionX, miny
					- world.player.PositionY);
		}
	}
}

