#include "world.h"
#include <cstdio>
#include <cstring>
#include <cmath>
#include <vector>
#undef min
#undef max
#include <algorithm>

static const double EPS = 1e-9;
static const double INF = 1e+50;
static const double DELTATIME = 1.0 / 60.0;

struct Point {
	double x, y;
	
	inline Point() : x(0), y(0) {}
	inline Point(double _x, double _y) : x(_x), y(_y) {}

	inline       double &operator[] (int i)       { return (i==0 ? x : y); }
	inline const double &operator[] (int i) const { return (i==0 ? x : y); }

	inline Point operator+ (const Point &b) const { return Point(x+b.x, y+b.y); }
	inline Point operator- (const Point &b) const { return Point(x-b.x, y-b.y); }
	inline Point operator* (double b) const { return Point(x*b, y*b); }
	inline Point operator/ (double b) const { return *this * (1.0/b); }

	inline void operator+= (const Point &b) { *this = *this + b; }
	inline void operator-= (const Point &b) { *this = *this - b; }
	inline void operator*= (double b) { *this = *this * b; }
	inline void operator/= (double b) { *this = *this / b; }

	inline double operator^ (const Point &b) const { return x*b.x + y*b.y; }
	inline double operator% (const Point &b) const { return x*b.y - y*b.x; }

	inline double operator!() const { return sqrt(*this ^ *this); }
};

Point Normal(const Point &x) {
	Point res = x;
	double norm = !res;
	if (norm > EPS) res /= norm;
	return res;
}

inline double trandom() { return rand() / 32768.0; }

#define POINT(name) Point(name##X, name##Y)
#define COORDS(pnt) (pnt).x, (pnt.y)
static const int MONSTERS = World::MAXMONSTERS;
static const int MISSILES = World::MAXMISSILES;

extern World *world;


static const double MOVE_RADIUS = 0.3;
static const double SPREADTIME = 0.1;

Point TargetMonsterDull(const CMonster &monster) {
	return POINT(monster.Position) - POINT(world->player.Position);
}

Point TargetMonsterMoving(const CMonster &monster) {
	Point pos = POINT(monster.Position) - POINT(world->player.Position);
	Point vel = POINT(monster.Velocity);
	pos += vel * (2.0*trandom()-1.0) * SPREADTIME;
	double speed = world->player.BulletVelocity;

	double qa = (vel ^ vel) - (speed * speed);
	double qb = (pos ^ vel);
	double qc = (pos ^ pos);
	if (qa > -EPS) return pos;
	double qd = qb*qb - qa*qc;
	qd = sqrt(qd);
	double t1 = (-qb - qd) / qa;
	double t2 = (-qb + qd) / qa;
	if (t1 < 0.0) t1 = INF;
	if (t2 < 0.0) t2 = INF;
	double t = std::min(t1, t2);

	return pos + vel*t;
}

void BotThink() {
	//Targeting
	std::vector<Point> arr;
	for (int i = 0; i<MONSTERS; i++) if (world->monsters[i].ClassName >= 0)
		arr.push_back(TargetMonsterMoving(world->monsters[i]));

	double mind = 1e+50;
	Point best;
	for (int i = 0; i<arr.size(); i++) if (mind > !arr[i]) {
		best = arr[i];
		mind = !best;
	}
//	Point best = (arr.size() ? arr[rand() % arr.size()] : Point());
    
	world->shoot(COORDS(best));

	//Moving
	Point radiusvec = POINT(world->player.Position) - Point(0.5, 0.5);
	double dist = !radiusvec;
	if (dist < EPS) radiusvec = Point(1, 0);
	else radiusvec = Normal(radiusvec);
	Point tangentvec(-radiusvec.y, radiusvec.x);

	Point force = tangentvec + radiusvec * (MOVE_RADIUS - dist) / MOVE_RADIUS;
	world->run(COORDS(force));
}

#ifndef BOTWITHGLUT
World *world;
int main(int argc, char** argv) {
	if (argc < 3) {
		fprintf(stderr, "Parameters: IP port\n");
		return 666;
	}
	world = new World(argv[1], atoi(argv[2]));
	world->makeMove();
	while (1) {
		BotThink();
		world->makeMove();
	}
	printf("Game over!\n");
	delete world;
	return 0;
}
#endif