#include "world.h"
#include <cstdio>
#include <cstring>
#include <cmath>

const double PI = 3.1415926535897932;
const double eps = 1e-8;

double dist2(double x1, double y1, double x2, double y2) {
	return (x1-x2)*(x1-x2)+(y1-y2)*(y1-y2);
}
int fl = 0;
World *world;
void circle(double &dx, double &dy) {
	const double coef = 1 / (0.4*PI);
	double tmp,alfa;
	if (world->player.PositionX > 0.9)
		fl = 1;
	if (!fl) {
		world->run(1,0);
		dx = 0;
		dy = 1;
	}
	else {
		alfa = acos(dx);
		if (dx < -eps)
			alfa = -alfa;
		dx = cos(alfa+coef);
		dy = sin(alfa+coef);
		tmp = sqrt(dx*dx+dy*dy);
		dx /= tmp;
		dy /= tmp;
		world->run(dx,dy);			
	}
}
void rect() {
	double x,y,dx,dy;
	if (world->player.PositionX > 0.6)
		fl = 1;
	if (!fl) {
		world->run(1,0);
		return;
	}
	x = world->player.PositionX - 0.5;
	y = world->player.PositionY - 0.5;
	if (x <= y && x > -y) {
		dx = 1;
		dy = 0;
	}
	else if (x < y && x <= -y) {
		dx = 0;
		dy = 1;
	}
	else if (x >=y && x < -y) {
		dx = -1;
		dy = 0;
	}
	else {
		dx = 0;
		dy = -1;
	}
	world->run(dx,dy);
}
void romb() {
	double dx,dy;
	const double coef = 0.95;
	double x,y;
	x = world->player.PositionX;
	y = world->player.PositionY;
	if (fl==0 && x >= coef)
		++fl;
	else if (fl==1 && y >= coef)
		++fl;
	else if (fl==2 && x <= 1.0-coef)
		++fl;
	else if (fl==3 && y <= 1.0-coef)
		++fl;
	else if (fl==4 && x >= coef)
		fl = 1;

	if (fl==0) {
		dx = 1;
		dy = 0;
	}
	else if (fl==1) {
		dx = -1;
		dy = 1;
	}
	else if (fl==2) {
		dx = -1;
		dy = -1;
	}
	else if (fl==3) {
		dx = 1;
		dy = -1;
	}
	else if (fl==4) {
		dx = 1;
		dy = 1;
	}
	world->run(dx,dy);
}
void octogon() {
	const double coef = 0.925;
	double x,y;
	x = world->player.PositionX;
	y = world->player.PositionY;
	if (!fl && x>=coef)
		fl = 1;
	else if (fl==1 && y>=0.65)
		fl = 2;
	else if (fl==2 && y>=coef)
		fl = 3;
	else if (fl==3 && x<=0.35)
		fl = 4;
	else if (fl==4 && x<=1-coef)
		fl = 5;
	else if (fl==5 && y<=0.35)
		fl = 6;
	else if (fl==6 && y<=1-coef)
		fl = 7;
	else if (fl==7 && x>=0.65)
		fl = 8;
	else if (fl==8 && x>=coef)
		fl = 1;
	double sh[9][2]={
		{1,0},
		{0,1},
		{-1,1},
		{-1,0},
		{-1,-1},
		{0,-1},
		{1,-1},
		{1,0},
		{1,1}
	};
	world->run(sh[fl][0],sh[fl][1]);
}
int two_wall_mode = 0;
void two_wall() {
	double x,y,dx,dy;
	x = world->player.PositionX;
	y = world->player.PositionY;
	if (fl==0 && fabs(x-0.5)<0.05 && fabs(y-0.95)<0.05)
		fl = 1;
	else if (fl==1 && y<=0.05) {
		two_wall_mode = 0;
		fl = 7;
	}
	if (fl==0) {
		dx = 0.5 - x;
		dy = 0.95 - y;
	}
	else if (fl==1) {
		dx = 0;
		dy = -1;
	}
	else {
		dx = dy = .0;
	}
	world->run(dx,dy);
}
double get_min_time(double xm, double ym, double vx, double vy, double vxp, double vyp, double px, double py) {
	double a,b,c,d,x1,y1,t;
	a = (vxp-vx)*(vxp-vx)+(vyp-vy)*(vyp-vy);
	b = (xm-px)*(vx-vxp) + (ym-py)*(vy-vyp);
	c = (xm-px)*(xm-px) + (ym-py)*(ym-py);
	d = b*b-a*c;
	if (d < -eps || fabs(a) < eps) {
		return -1;
	}
	if (d < eps)
		d = 0;
	t = (-b - sqrt(d)) / a;
	if (t < -eps)
		return -1;
	x1 = xm + t * vx;
	y1 = ym + t * vy;
	if (x1 < -eps || y1 < -eps || x1 - 1 > eps || y1 - 1 > eps)
		return -1;
	return t;
}
void get_coord_brute(double xm, double ym, double vx, double vy) {
	double bv = world->player.BulletVelocity;
	double px = world->player.PositionX;
	double py = world->player.PositionY;
	double alfa, step = .001,vxp,vyp,t,aa;
	double tt = 1e10;
	for (alfa=.0;alfa < 2*PI;alfa+=step) {
		vxp = bv * cos(alfa);
		vyp = bv * sin(alfa);
		t = get_min_time(xm,ym,vx,vy,vxp,vyp,px,py);
		if (t!=-1 && t - tt < -eps) {
			tt = t;
			aa = alfa;
		}
	}
	double ax, ay;
	ax = bv * cos(aa);
	ay = bv * sin(aa);
	world->shoot(ax,ay);
}
void get_coord_stupid(double mx, double my, double x, double y) {
	x = mx - x;
	y = my - y;
	world->shoot(x,y);
}
bool kill_bullet() {
	int i,pos = -1;
	double t,tt=1e10;
	for (i=0;i<world->MAXMISSILES;++i) {
		if (world->missiles[i].ClassName>0) {
			t = get_min_time(world->player.PositionX,world->player.PositionY,
				world->player.VelocityX,world->player.VelocityY,
				world->missiles[i].VelocityX,world->missiles[i].VelocityY,
				world->missiles[i].PositionX,world->missiles[i].PositionY);
			if (t!=-1 && t - tt < -eps) {
				tt = t;
				pos = i;
			}
		}
	}
	if (tt > 1/6.0)
		return false;
	get_coord_brute(world->missiles[pos].PositionX,world->missiles[pos].PositionY,
					world->missiles[pos].VelocityX,world->missiles[pos].VelocityY);
	return true;
}
int *use;
const int max_shoot = 10;
int get_beaten_monster() {
	int pos = -1;
	double d,tmp;
	for ( int j = 0; j < world->MAXMONSTERS; j++) {
		if (world->monsters[j].ClassName >= 0) {
			tmp = dist2(world->monsters[j].PositionX, world->monsters[j].PositionY,  world->player.PositionX,  world->player.PositionY);
			if ((use[j] < max_shoot || tmp < 0.1) && (pos==-1 || world->monsters[j].HitPoints - d < -eps)) {
				d = world->monsters[j].HitPoints;
				pos = j;
			}
		}
		else {
			use[j] = 0;
		}
	}
	return pos;
}
int main(int argc, char** argv) {
	//Check for enough parameters
	if (argc < 3) {
		fprintf(stderr, "Parameters: IP port\n");
		return 10;
	}

	//Create the world data (and establish connection with the server)
	world = new World(argv[1], atoi(argv[2]));
	double d,tmp,d2;

	int pos;
	//Infinite loop
	use = new int[world->MAXMONSTERS];
	double dx,dy,ax,ay,cs;
	int it = 0,pos2;
	while (true) {
		++it;
		//Send our turn and receive the update of the game world
		world->makeMove();
		//Print some information
		printf("frame : %d, HP: %.2lf x: %.2lf y:%.2lf\n",
				world->player.Time, world->player.HitPoints,
				world->player.PositionX, world->player.PositionY);
		
		//circle(dx,dy);
		//rect();
		//romb();
		
		if (it==3200) {
			fl = 0;
			two_wall_mode = 1;
		}
		if (two_wall_mode==1)
			two_wall();
		else octogon();
		
		if (it>4200) {
			world->run(1-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 4500) {
			world->run(-1,0);
		}
		if (it > 5000) {
			world->run(1-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 5400) {
			world->run(0-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 5800) {
			world->run(1-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 6200) {
			world->run(0-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 6600) {
			world->run(1-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 7000) {
			world->run(0-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 7400) {
			world->run(1-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 7800) {
			world->run(0-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 8200) {
			world->run(1-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 8600) {
			world->run(0-world->player.PositionX,1-world->player.PositionY);
		}
		if (it > 9000) {
			world->run(1-world->player.PositionX,1-world->player.PositionY);
		}
		

		
		
		//Choose the first monster that is still alive
		pos = -1;
		for ( int j = 0; j < world->MAXMONSTERS; j++) {
			if (world->monsters[j].ClassName >= 0 && use[j] < max_shoot) {
				pos = 0;
				break;
			}
		}
		if (pos==-1) {
			for ( int j = 0; j < world->MAXMONSTERS; j++) {
				use[j] = 0;
			}
		}
		pos = -1;
		pos2 = -1;
		for ( int j = 0; j < world->MAXMONSTERS; j++) {
			if (world->monsters[j].ClassName >= 0) {
				tmp = dist2(world->monsters[j].PositionX, world->monsters[j].PositionY,  world->player.PositionX,  world->player.PositionY);
				cs = (world->monsters[j].PositionX  - world->player.PositionX) * world->player.VelocityX;
				cs += (world->monsters[j].PositionY  - world->player.PositionY) * world->player.VelocityY;
				if ((use[j] < max_shoot || tmp < 0.1) && (pos==-1 || tmp - d < -eps)) {
					d = tmp;
					pos = j;
				}
				if (cs > eps && (pos2==-1 || tmp - d2 < -eps)) {
					d2 = tmp;
					pos2 = j;
				}
			}
			else {
				use[j] = 0;
			}
		}
		if (pos2!=-1 && it < 1750 )
			pos = pos2;
		//if (it>1500 && it<1800) {
		//	pos = get_beaten_monster();
		//}
		if (pos==-1)
			pos = 0;
		++use[pos];
		//Shoot the bastard!
		if (it < 1750)
			get_coord_stupid(world->monsters[pos].PositionX, world->monsters[pos].PositionY, world->player.PositionX,  world->player.PositionY);
		else if (it < 3100)
			get_coord_brute(world->monsters[pos].PositionX,world->monsters[pos].PositionY,world->monsters[pos].VelocityX,world->monsters[pos].VelocityY);
		else if (it < 5250)
			get_coord_stupid(world->monsters[pos].PositionX, world->monsters[pos].PositionY, world->player.PositionX,  world->player.PositionY);
		else {
			get_coord_brute(world->monsters[pos].PositionX,world->monsters[pos].PositionY,world->monsters[pos].VelocityX,world->monsters[pos].VelocityY);
			//get_coord_stupid(world->monsters[pos].PositionX, world->monsters[pos].PositionY, world->player.PositionX,  world->player.PositionY);
			//if (!kill_bullet()) {
			//	get_coord_stupid(world->monsters[pos].PositionX, world->monsters[pos].PositionY, world->player.PositionX,  world->player.PositionY);
			//}
		}
	}

	return 0;
}
