import java.io.IOException;
import java.util.Arrays;

class Task {
	public static void main(String[] args) throws IOException {

		// Check for enough parameters
		if (args.length < 2) {
			System.err.println("parameters: IP port");
			System.exit(10);
		}

		// Create the world data (and establish connection with the server)
		MyWorld world = new MyWorld(args[0], Integer.parseInt(args[1]));
		world.start();
	}

	static class MyWorld extends World {
		public MyWorld(String server, int port) {
			super(server, port);
		}

		void run(double x, double y) {
			run((float) x, (float) y);
		}

		void shoot(double x, double y) {
			shoot((float) x, (float) y);
		}

		int level = 1;

		void makeMoveLevel1() {
			double a = 20;// 1;//.3; //monsters
			double b = 400;// 100; // rotation
			double c = 60; // circle
			double mis = 1; // missiles

			double rad = 0.3;

			double gradX = 0;
			double gradY = 0;

			int mcnt = 0;

			for (int j = 0; j < MAXMONSTERS; ++j) {
				if (monsters[j].ClassName < 0)
					continue;

				mcnt++;
				double dd = distBounded(player.PositionX, player.PositionY, monsters[j].PositionX, monsters[j].PositionY);
				if (dd > .3) {
					continue;
				}

				dd = dd * dd;

				gradX += a * (player.PositionX - monsters[j].PositionX) / dd;
				gradY += a * (player.PositionY - monsters[j].PositionY) / dd;

			}

			if (mcnt > 170) {
				level = 3;
				makeMoveLevel3();
				return;
			}

			double R = .1 + 0.25 * mcnt / 300;
			// for (int j = 0; j < MAXMISSILES; ++j) {
			// if (missiles[j].ClassName < 0) continue;
			//
			// double dd = distBounded(player.PositionX, player.PositionY,
			// missiles[j].PositionX, missiles[j].PositionY);
			// dd = dd * dd * dd * dd;
			//
			// gradX += mis * (player.PositionX - missiles[j].PositionX) / dd;
			// gradY += mis * (player.PositionY - missiles[j].PositionY) / dd;
			//
			// }

			double ttd = dist(.5, .5, player.PositionX, player.PositionY);

			if (ttd > rad) {
				gradX -= b * Math.sqrt(mcnt) * (.5 - player.PositionY) / distBounded(.5, .5, player.PositionX, player.PositionY);
				gradY += b * Math.sqrt(mcnt) * (.5 - player.PositionX) / distBounded(.5, .5, player.PositionX, player.PositionY);
			}

			double x1 = player.PositionX - .5;
			double y1 = player.PositionY - .5;
			double d = Math.sqrt(x1 * x1 + y1 * y1);
			double x2 = x1 * R / d;
			double y2 = y1 * R / d;

			if (!Double.isNaN(x2)) {
				gradX += c * Math.sqrt(mcnt) * (x2 - x1) * dist(x1, y1, x2, y2);
				gradY += c * Math.sqrt(mcnt) * (y2 - y1) * dist(x1, y1, x2, y2);
			}

			run(gradX, gradY);
		}

		void makeMoveLevel3() {
			double a = .5;// 1;//.3;
			double b = 10;// 100;
			double c = 50;
			double rad = .25;
			double R = .25;
			double gradX = 0;
			double gradY = 0;

			double mis = 1;
			
			int mcnt = 0;

			for (int j = 0; j < MAXMONSTERS; ++j) {
				if (monsters[j].ClassName < 0)
					continue;

				double dd = distBounded(player.PositionX, player.PositionY, monsters[j].PositionX, monsters[j].PositionY);
				dd = dd * dd;

				gradX += a * (player.PositionX - monsters[j].PositionX) / dd;
				gradY += a * (player.PositionY - monsters[j].PositionY) / dd;
				mcnt++;
			}
			
			
			
			for (int j = 0; j < MAXMISSILES; ++j) {
				if (missiles[j].ClassName <= 200)
					continue;

				double dd = distBounded(player.PositionX, player.PositionY, missiles[j].PositionX, missiles[j].PositionY);
				
				if (dd > .3) {
					continue;
				}
				
				dd = dd * dd;

				gradX += mis * (player.PositionX - missiles[j].PositionX) / dd;
				gradY += mis * (player.PositionY - missiles[j].PositionY) / dd;

			}
			

			if (mcnt < 30) {
				level = 4;
			}

			double ttd = dist(.5, .5, player.PositionX, player.PositionY);

			if (ttd > rad) {
				gradX -= b * (Math.sqrt(mcnt) + 5) * (.5 - player.PositionY) / distBounded(.5, .5, player.PositionX, player.PositionY);
				gradY += b * (Math.sqrt(mcnt) + 5) * (.5 - player.PositionX) / distBounded(.5, .5, player.PositionX, player.PositionY);
			}

			double x1 = player.PositionX - .5;
			double y1 = player.PositionY - .5;
			double d = Math.sqrt(x1 * x1 + y1 * y1);
			double x2 = x1 * R / d;
			double y2 = y1 * R / d;

			if (!Double.isNaN(x2)) {
				gradX += c * (Math.sqrt(mcnt) + 5) * (x2 - x1);
				gradY += c * (Math.sqrt(mcnt) + 5) * (y2 - y1);
			}

			run(gradX, gradY);
		}

		void makeMoveLevel4() {
			
//			double a = .5;// 1;//.3;
//			double b = 10;// 100;
//			double c = 50;
//			double rad = .25;
//			double R = .25;
			
			double a = .5;// 1;//.3;
			double b = 100;// 100;
			double c = 500;
			double rad = .25;
			double R = .35;

			double mis = 1; // missiles

			double gradX = 0;
			double gradY = 0;

			int mcnt = 0;

			for (int j = 0; j < MAXMONSTERS; ++j) {
				if (monsters[j].ClassName < 0)
					continue;

				double dd = distBounded(player.PositionX, player.PositionY, monsters[j].PositionX, monsters[j].PositionY);
				if (dd > .3) {
					continue;
				}
				dd = dd * dd;

				gradX += a * (player.PositionX - monsters[j].PositionX) / dd;
				gradY += a * (player.PositionY - monsters[j].PositionY) / dd;
				mcnt++;
			}

			for (int j = 0; j < MAXMISSILES; ++j) {
				if (missiles[j].ClassName <= 200)
					continue;

				double dd = distBounded(player.PositionX, player.PositionY, missiles[j].PositionX, missiles[j].PositionY);
				dd = dd * dd;

				gradX += mis * (player.PositionX - missiles[j].PositionX) / dd;
				gradY += mis * (player.PositionY - missiles[j].PositionY) / dd;

			}

			double ttd = dist(.5, .5, player.PositionX, player.PositionY);

			if (ttd > rad) {
				gradX -= b * (Math.sqrt(mcnt) + 5) * (.5 - player.PositionY) / distBounded(.5, .5, player.PositionX, player.PositionY);
				gradY += b * (Math.sqrt(mcnt) + 5) * (.5 - player.PositionX) / distBounded(.5, .5, player.PositionX, player.PositionY);
			}

			double x1 = player.PositionX - .5;
			double y1 = player.PositionY - .5;
			double d = Math.sqrt(x1 * x1 + y1 * y1);
			double x2 = x1 * R / d;
			double y2 = y1 * R / d;

			if (!Double.isNaN(x2)) {
				gradX += c * (Math.sqrt(mcnt) + 5) * (x2 - x1);
				gradY += c * (Math.sqrt(mcnt) + 5) * (y2 - y1);
			}

			run(gradX, gradY);
		}

		double eps = 1e-6;

		void shootFast() {
			// if (shootBullet()) {
			// System.out.println("shoot bullet " + player.Time);
			// return;
			// }
			shootMonster();
		}

		boolean shootMonster() {
			if (player.GunCooldownRemaining > 1.0 / 60) {
				return false;
			}
			double tBest = Double.POSITIVE_INFINITY;
			double ax = 1;
			double ay = 0;
			int mm = -1;
			double mt = 0;
			for (int j = 0; j < MAXMONSTERS; j++) {
				if (monsters[j].ClassName < 0)
					continue;
				if (monsterKillTime[j] > player.Time / 60.0) {
					continue;
				}
				if (monsterExpHit[j] < 0 && monsters[j].HitPoints > 0) {
					monsterExpHit[j] = monsters[j].HitPoints;
				}
				double vx = monsters[j].VelocityX;
				double vy = monsters[j].VelocityY;
				double x0 = monsters[j].PositionX;
				double y0 = monsters[j].PositionY;
				double t = getHitTime(x0, y0, vx, vy);
				int steps = 2;
				double tt = getHitTime(x0 + (vx - player.VelocityX) * steps / 60.0, y0 + (vy - player.VelocityY) * steps / 60.0, vx, vy);
				double ttt = player.GunCooldownPeriod * (Math.ceil(monsterExpHit[j] / player.BulletDamage) - 1);
				if (3 * tt + ttt < tBest) {
					tBest = 3 * tt + ttt;
					x0 += player.GunCooldownRemaining * vx;
					y0 += player.GunCooldownRemaining * vy;
					double x1 = player.PositionX + player.GunCooldownRemaining * player.VelocityX;
					double y1 = player.PositionY + player.GunCooldownRemaining * player.VelocityY;
					double xk = x0 + vx * t;
					double yk = y0 + vy * t;
					ax = xk - x1;
					ay = yk - y1;
					mm = j;
					mt = t;
				}
			}
			// System.err.println(tBest);
			if (mm > -1) {
				shoot(ax, ay);
				monsterExpHit[mm] -= player.BulletDamage;
				if (monsterExpHit[mm] < 0) {
					monsterKillTime[mm] = player.Time / 60.0 + mt;
					monsterExpHit[mm] = Double.NEGATIVE_INFINITY;
				}
				System.err.println(mm + " - expHit:" + monsterExpHit[mm] + " mkt:" + monsterKillTime[mm] + "  ct: " + player.Time / 60.0);
				return true;
			}
			return false;
		}

		boolean shootBullet() {
			if (player.GunCooldownRemaining > 1.0 / 60) {
				return false;
			}
			double tBest = Double.POSITIVE_INFINITY;
			double ax = 1;
			double ay = 0;
			int mm = -1;
			for (int j = 0; j < MAXMISSILES; j++) {
				if (missiles[j].ClassName <= 200)
					continue;
				double vx = missiles[j].VelocityX;
				double vy = missiles[j].VelocityY;
				double x0 = missiles[j].PositionX;
				double y0 = missiles[j].PositionY;
				double t = getHitTime(x0, y0, vx, vy);
				if (t < tBest) {
					tBest = t;
					x0 += player.GunCooldownRemaining * vx;
					y0 += player.GunCooldownRemaining * vy;
					double x1 = player.PositionX + player.GunCooldownRemaining * player.VelocityX;
					double y1 = player.PositionY + player.GunCooldownRemaining * player.VelocityY;
					double xk = x0 + vx * t;
					double yk = y0 + vy * t;
					ax = xk - x1;
					ay = yk - y1;
					mm = j;
				}
			}
			// System.err.println(tBest);
			if (mm > -1) {
				shoot(ax, ay);
				return true;
			}
			return false;
		}

		double getHitTime(double x0, double y0, double vx, double vy) {
			double x1 = player.PositionX;
			double y1 = player.PositionY;
			double u = player.BulletVelocity;
			x0 += player.GunCooldownRemaining * vx;
			y0 += player.GunCooldownRemaining * vy;
			x1 += player.GunCooldownRemaining * player.VelocityX;
			y1 += player.GunCooldownRemaining * player.VelocityY;
			double dx = x0 - x1;
			double dy = y0 - y1;
			double A = vx * vx + vy * vy - u * u;
			double B = dx * vx + dy * vy;
			double C = dx * dx + dy * dy;
			double t = Double.POSITIVE_INFINITY;
			if (Math.abs(A) > eps) {
				double D = B * B - A * C;
				if (D > -eps) {
					D = Math.sqrt(Math.abs(D));
					double t1 = (-B - D) / A;
					if (t1 > eps) {
						t = Math.min(t, t1);
					}
					double t2 = (-B + D) / A;
					if (t2 > eps) {
						t = Math.min(t, t2);
					}
				}
			} else if (Math.abs(B) > eps) {
				double t1 = -C / (2 * B);
				if (t1 > eps) {
					t = Math.min(t, t1);
				}
			} else {
			}
			t += player.GunCooldownRemaining;
			return t;
		}

		int countMonsters() {

			boolean[] u = new boolean[1000];

			int ans = 0;
			for (int j = 0; j < MAXMONSTERS; ++j) {
				int cn = monsters[j].ClassName;
				if (monsters[j].ClassName >= 0) {
					if (!u[cn]) {
//						System.out.print(cn + " ");
						u[cn] = true;
					}
					++ans;
				}
			}
			return ans;
		}

		double[] monsterKillTime;
		double[] monsterExpHit;

		void start() {
			monsterKillTime = new double[MAXMONSTERS];
			monsterExpHit = new double[MAXMONSTERS];
			Arrays.fill(monsterKillTime, -1);
			Arrays.fill(monsterExpHit, Double.NEGATIVE_INFINITY);

			while (true) {
				// Send our turn and receive the update of the game world
				makeMove();

//				System.out.print("lvl: " + level + " ");
//				countMonsters();
//				System.out.println();
				if (level == 1) {
					makeMoveLevel1();
				} else if (level == 3) {
					makeMoveLevel3();
				} else {
					makeMoveLevel3();
				}
				shootFast();
			}
		}

		double distBounded(double x1, double y1, double x2, double y2) {
			return Math.max(Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)), .01);
		}

		double dist(double x1, double y1, double x2, double y2) {
			return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
		}

		double dist(int j) {
			double x1 = player.PositionX;
			double y1 = player.PositionY;
			double x2 = monsters[j].PositionX;
			double y2 = monsters[j].PositionY;
			return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
		}
	}
}