#include "world.h"
#include <cstdio>
#include <cstring>
#include <cmath>
#include <algorithm>
#include <fstream>
#include <vector>

const double M_PI=3.1415926535897932384626433832795;

struct point
{
	double x,y;
	inline friend point operator - (point a,point b)
	{
		point ans;
		ans.x=a.x-b.x;
		ans.y=a.y-b.y;
		return ans;
	};
	point(double X,double Y)
	{
		x=X;
		y=Y;
		return;
	};
	point(double angle)
	{
		x=cos(angle);
		y=sin(angle);
		return;
	};
	point(void)
	{
		point(0,0);
		return;
	};
};

using namespace std;


inline double sqr(double x);
inline double dist(point a,point b);
inline double shiftRandomAngle(double minAngle,double maxAngle);
inline point getPoint(CMonster monster);
inline pair<point,double> shootWithDelay(point playerPosition,point monsterPosition,point monsterVelocity,double missleVelocity);
inline int getNearestMonster(World *world,vector<double> &HP);
inline point getShootVector(World *world,int &currentMonster,vector<double> &HP,vector<double> &Time);
inline point getDirection(World *world,int &currentDirection,double &alpha);
inline pair<int,int> getNextEnemy(World *world,vector<double> &HP);
inline double scalar(point a,point b);
inline double vectorP(point a,point b);
inline double angleBetweenVectors(point a,point b);
inline int getQuantityOfMonstersInSector(World *world,double alpha,double r,vector<double> &HP);
inline double angleToEnemy(point us,point monster,point velocity);
inline pair<int,int> getNearestInSector(World *world,double alpha,double r,vector<double> &HP);
inline int getNearestShooter(World *world,vector<double> &HP);
inline void reloadMonsters(World *world,vector<double>&HP);
inline int quantityOfMonsters(World *world,vector<double>&HP);


bool was=false,isShooters=false;
ofstream fout("output.txt");

inline double scalar(point a,point b)
{
	return a.x*b.x+a.y*b.y;
}

inline double vectorP(point a,point b)
{
	return a.x*b.y-a.y*b.x;
}

inline double angleBetweenVectors(point a,point b)
{
	return atan2(vectorP(a,b),scalar(a,b));
}

inline double sqr(double x)
{
	return x*x;
}

inline double shiftRandomAngle(double minAngle,double maxAngle)
{
	return minAngle+rand()*1.0/RAND_MAX*(maxAngle-minAngle);
}

inline point getPoint(CMonster monster)
{
	return point(monster.PositionX,monster.PositionY);
}

inline double dist(point a,point b)
{
	return sqrtl((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y));
}

inline pair<point,double>shootWithDelay(point playerPosition,point monsterPosition,point monsterVelocity,double missleVelocity)
{
	double A=missleVelocity*( monsterPosition.y-playerPosition.y);
	double B=missleVelocity*(-monsterPosition.x+playerPosition.x);
	double C=monsterVelocity.x*(monsterPosition.y-playerPosition.y)-
		     monsterVelocity.y*(monsterPosition.x-playerPosition.x);
	double D=C/sqrtl(sqr(A)+sqr(B));
	double angle=atan2(A,B);
	double phi1=asin(D);
	double phi2=M_PI-asin(D);
	double alpha1=phi1-angle;
	double alpha2=phi2-angle;
	double t1=(monsterPosition.x-playerPosition.x)/(missleVelocity*cos(alpha1)-monsterVelocity.x);
	double t2=(monsterPosition.x-playerPosition.x)/(missleVelocity*cos(alpha2)-monsterVelocity.x);
	if(t1>t2)
	{
		swap(t1,t2);
		swap(alpha1,alpha2);
	}
	if(t1>=0)
	{
		fout<<cos(alpha1)<<' '<<sin(alpha1)<<' '<<t1<<endl;
		return make_pair(point(alpha1),t1*60.0);
	}
	else
		if(t2>=0)
		{
			fout<<cos(alpha2)<<' '<<sin(alpha2)<<' '<<t2<<endl;
			return make_pair(point(alpha2),t2*60.0);
		}
	return make_pair(point(monsterPosition.x-playerPosition.x,monsterPosition.y-playerPosition.y),0);
}

inline int quantityOfMonsters(World *world,vector<double>&HP)
{
	int ans=0;
	for(int i=0;i<world->MAXMONSTERS;i++)
		if(world->monsters[i].ClassName>=0&&HP[i]>0)
			return 1;
	return 0;
}

inline void reloadMonsters(World *world,vector<double>&HP)
{
	for(int i=0;i<world->MAXMONSTERS;i++)
		if(world->monsters[i].ClassName>=0)
			HP[i]=world->monsters[i].HitPoints;
	return;
}

inline int getNearestMonster(World *world,vector<double> &HP)
{
	if(quantityOfMonsters(world,HP)==0)
		reloadMonsters(world,HP);
	double ourX=world->player.PositionX,ourY=world->player.PositionY;
	point us=point(ourX+world->player.VelocityX*0.3,ourY+world->player.VelocityX*0.3);
	point ans=point(-1000,-1000);
	int best=-1;
	for(int i=0;i<world->MAXMONSTERS;i++)
		if(world->monsters[i].ClassName>=0)
			if(dist(us,ans)>dist(us,getPoint(world->monsters[i])))
				if(world->monsters[i].BulletDamage==0)
				{
					if(HP[i]>0||!isShooters)
					{
						ans=getPoint(world->monsters[i]);
						best=i;
					}
				}
				else
					isShooters=true;
	if(best==-1)
		best=getNearestShooter(world,HP);
	return best;
}

inline int getNearestShooter(World *world,vector<double> &HP)
{
	if(was)
	{
		was=false;
		return 0;
	}
	was=true;
	if(quantityOfMonsters(world,HP)==0)
		reloadMonsters(world,HP);
	double ourX=world->player.PositionX,ourY=world->player.PositionY;
	point us=point(ourX+world->player.VelocityX*0.3,ourY+world->player.VelocityX*0.3);
	point ans=point(-1000,-1000);
	int best=-1;
	isShooters=false;
	for(int i=0;i<world->MAXMONSTERS;i++)
		if(world->monsters[i].ClassName>=0)
			if(dist(us,ans)>dist(us,getPoint(world->monsters[i])))
				if(world->monsters[i].BulletDamage>0)
				{
					isShooters=true;
					if(HP[i]>0)
					{
						ans=getPoint(world->monsters[i]);
						best=i;
					}
				}
	if(best==-1)
		best=getNearestMonster(world,HP);
	was=false;
	return best;
}

inline double angleToEnemy(point us,point monster,point velocity)
{
	return angleBetweenVectors(monster-us,velocity);
}

inline int getQuantityOfMonstersInSector(World *world,double alpha,double r,vector<double> &HP)
{
	double ourX=world->player.PositionX,ourY=world->player.PositionY;
	point us=point(ourX,ourY);
	point ourVelocity=point(world->player.VelocityX,world->player.VelocityY);
	int ans=0;
	for(int i=0;i<world->MAXMONSTERS;i++)
		if(world->monsters[i].ClassName>=0)
			if(r>dist(us,getPoint(world->monsters[i]))&&fabs(angleToEnemy(us,getPoint(world->monsters[i]),ourVelocity))<alpha)
				if(HP[i]>0)
					ans++;
	return ans;
}

inline pair<int,int>getNearestInSector(World *world,double alpha,double r,vector<double> &HP)
{
	double ourX=world->player.PositionX,ourY=world->player.PositionY;
	point us=point(ourX+world->player.VelocityX*0.3,ourY+world->player.VelocityY*0.3);
	point ans=point(-1000,-1000);
	point ourVelocity=point(world->player.VelocityX,world->player.VelocityY);
	int best=0;
	for(int i=0;i<world->MAXMONSTERS;i++)
		if(world->monsters[i].ClassName>=0)
			if(r>dist(us,getPoint(world->monsters[i]))&&fabs(angleToEnemy(us,getPoint(world->monsters[i]),ourVelocity))<alpha)
				if(HP[i]>0)
					if(dist(us,ans)>dist(us,getPoint(world->monsters[i])))
					{
						ans=getPoint(world->monsters[i]);
						best=i;
					}
	return make_pair(best,0);
}

inline pair<int,int>getNextEnemy(World *world,vector<double> &HP)
{
	/*for(int i=0;i<world->MAXMISSILES;i++)
		if(world->missiles[i].ClassName>=0)
			return make_pair(i,1);*/
	if(1.0*rand()*1.0/RAND_MAX<0.2||!isShooters)
		return make_pair(getNearestMonster(world,HP),0);
	return make_pair(getNearestShooter(world,HP),0);
}

inline point getShootVector(World *world,int &currentMonster,vector<double> &HP,vector<double> &Time)
{
	point us=point(world->player.PositionX,world->player.PositionY);
	pair<int,int>nearestEnemy=getNextEnemy(world,HP);
	if(getQuantityOfMonstersInSector(world,M_PI/4,0.3,HP)>3)
		nearestEnemy=getNearestInSector(world,M_PI/4,0.3,HP);
	if(nearestEnemy.second==0)
	{
		point enemyPosition=point(world->monsters[nearestEnemy.first].PositionX,world->monsters[nearestEnemy.first].PositionY);
		point enemyVelocity=point(world->monsters[nearestEnemy.first].VelocityX,world->monsters[nearestEnemy.first].VelocityY);
		pair<point,double> ans=shootWithDelay(us,enemyPosition,enemyVelocity,world->player.BulletVelocity);
		HP[nearestEnemy.first]-=0.8*world->player.BulletDamage;
		if(HP[nearestEnemy.first]<0)
			Time[nearestEnemy.first]=world->player.Time+ans.second;
		return ans.first;
	}
	else
	{
		point enemyPosition=point(world->missiles[nearestEnemy.first].PositionX,world->missiles[nearestEnemy.first].PositionY);
		point enemyVelocity=point(world->missiles[nearestEnemy.first].VelocityX,world->missiles[nearestEnemy.first].VelocityY);
		return(shootWithDelay(us,enemyPosition,enemyVelocity,world->player.BulletVelocity)).first;
	}
}

inline point getDirection(World *world,int &currentDirection,double &alpha)
{
	if(world->player.Time<4500)
	{
		if(currentDirection==0&&world->player.PositionX>=0.88)
			currentDirection=1;
		if(currentDirection==2&&world->player.PositionX<=0.12)
			currentDirection=3;
		if(currentDirection==1&&world->player.PositionY>=0.88)
			currentDirection=2;
		if(currentDirection==3&&world->player.PositionY<=0.12)
			currentDirection=0;
		if(currentDirection==0)
			return point(1,0);
		if(currentDirection==2)
			return point(-1,0);
		if(currentDirection==1)
			return point(0,1);
		return point(0,-1);
	}
	else
	{
		isShooters=false;
		return point();
	}
}

int main(int argc, char** argv) {
	//Check for enough parameters
	if (argc < 3) {
		fprintf(stderr, "Parameters: IP port\n");
		return 10;
	}

	//Create the world data (and establish connection with the server)
	World *world = new World(argv[1], atoi(argv[2]));
	int direction=3;
	int currentMonster=0;

	vector<double>HP(world->MAXMONSTERS);
	vector<double>Time(world->MAXMONSTERS,-1000);

	double alpha=0;

	//Infinite loop
	while (true) {
		/*//Send our turn and receive the update of the game world
		world->makeMove();

		//Print some information
		printf("frame : %d, HP: %.2lf x: %.2lf y:%.2lf\n",
				world->player.Time, world->player.HitPoints,
				world->player.PositionX, world->player.PositionY);

		alpha += 0.02;
		//Run in circles (set direction)
		world->run(cos(alpha), sin(alpha));

		//Choose the first monster that is still alive
		for ( int j = 0; j < world->MAXMONSTERS; j++)
			if (world->monsters[j].ClassName >= 0) {
				//Shoot the bastard! (Shoot in the direction of the monster)
				world->shoot(world->monsters[j].PositionX - world->player.PositionX,
						   world->monsters[j].PositionY - world->player.PositionY);
				break;
			}*/
		world->makeMove();
		for(int i=0;i<world->MAXMONSTERS;i++)
			if(Time[i]<world->player.Time&&world->monsters[i].ClassName>=0)
				HP[i]=world->monsters[i].HitPoints;
		point dir=getDirection(world,direction,alpha);
		world->run(dir.x,dir.y);
		point toShoot=getShootVector(world,currentMonster,HP,Time);
		world->shoot(toShoot.x,toShoot.y);
		printf("frame : %d, HP: %.2lf x: %.2lf y:%.2lf\n",
				world->player.Time, world->player.HitPoints,
				world->player.PositionX, world->player.PositionY);
	}

	return 0;
}
