#include "world.h"
#include <cstdio>
#include <cstring>
#include <cmath>
#include <iostream>
#include <vector>

using namespace std;

float calcT(float x, float y, float vel, float toX, float toY)
{
	return hypot(x-toX, y-toY)/vel;
}

int main(int argc, char** argv) {
	//Check for enough parameters
	if (argc < 3) {
		fprintf(stderr, "Parameters: IP port\n");
		return 10;
	}

	//Create the world data (and establish connection with the server)
	World *world = new World(argv[1], atoi(argv[2]));
	float alpha = 0.0;

	vector<int> shots(256,0);
	vector<float> lastX(256,0), lastY(256,0), lastTime(256,0);

	float time = 0.0f;

	//Infinite loop
	while (true) {
		//Send our turn and receive the update of the game world
		world->makeMove();

		float cx = 0, cy = 0;

		int cnt = 0;
		float hl = 0;

		float px = world->player.PositionX;
		float py = world->player.PositionY;

		int nearest = -1;
		float bestDist;

		for(int i=0; i<World::MAXMONSTERS; ++i)
		{
			if (world->monsters[i].ClassName != -1)
			{
				float mx = world->monsters[i].PositionX;
				float my = world->monsters[i].PositionY;

				float dist = hypot(px - mx, py - my);

				if ((nearest == -1 || dist < bestDist) && shots[i]*world->player.BulletDamage < 2*world->monsters[i].HitPoints)
				{
					nearest = i;
					bestDist = dist;
				}

				cnt++;

				if (hypot(mx-px, my-py) < 0.4f)
				{
					float health = (world->monsters[i].HitPoints / world->monsters[i].MaxHitPoints);
					hl += health;
					cx += health * mx;
					cy += health * my;
				}
			}
		}

		float mx = world->monsters[nearest].PositionX;
		float my = world->monsters[nearest].PositionY;

		if (cnt == 0) continue;

		shots[nearest]++;
		cout << "shots " << shots[nearest] << endl;

		cx/=(hl+1E-6);
		cy/=(hl+1E-6);

		float powerX = -2*(px - 0.5f);
		float powerY = -2*(py - 0.5f);

		float powerLen = hypot(powerX, powerY);

		float drunX = px - cx;
		float drunY = py - cy;

		float runX = drunX/(hypot(drunX, drunY) + 1E-2);
		float runY = drunY/(hypot(drunX, drunY) + 1E-2);

		runX += powerX;
		runY += powerY;
			
		float runLen = hypot(runX,runY);
		if (runLen < 2*1E-1f)// && runLen/(powerLen+1E-4f) < 1E-1f)
		{
			//cout << "!\n";
			runX = -drunY;
			runY = drunX;
		}

		world->run(runX, runY);

		///bin search
		float a=0.0f, b=5.0f;
		float monVx = world->monsters[nearest].VelocityX;
		float monVy = world->monsters[nearest].VelocityY;

		float fa = calcT(px, py, world->player.BulletVelocity, mx + a*monVx, my + a*monVy);
		float fb = calcT(px, py, world->player.BulletVelocity, mx + b*monVx, my + b*monVy);

		while(1)
		{
			float mid = (a+b)/2.0f;
			float fmid = calcT(px, py, world->player.BulletVelocity, mx + mid*monVx, my + mid*monVy);

			if (fmid > mid)
			{
				a = mid;
				fa = fmid;
			}
			else
			{
				b = mid;
				fb = fmid;
			}

			if (fabs(a-b) < 1E-5f)
				break;
		}
		///bin search

		float t = (a+b)/2.0f;
		float msPosX = mx + t*monVx;
		float msPosY = my + t*monVy;

		world->shoot(	msPosX - px, 
						msPosY - py);

	//Print some information
		printf("frame : %d, HP: %.2lf x: %.2lf y:%.2lf %.2lf %.2lf %.2lf %.2lf\n",
				world->player.Time, 
				world->player.HitPoints,
				px, py, runX, runY, msPosX - px, msPosY - py);

		lastTime[nearest] = time + t;

		for(int i=0; i<World::MAXMONSTERS; ++i)
		{
			if (world->monsters[i].ClassName != -1)
			{
				if ((hypot(lastX[i]-world->monsters[i].PositionX, lastY[i]-world->monsters[i].PositionY)
					 > 1.0f/60.0f * hypot(world->monsters[i].VelocityX, world->monsters[i].VelocityY)) + 1E-4 )
				{
					//cout << "cond1\n";
					shots[i] = 0;
				}

				if (time > lastTime[i])
				{
					//cout << "cond2\n";
					shots[i] = 0;
				}

				lastX[i] = world->monsters[i].PositionX;
				lastY[i] = world->monsters[i].PositionY;
			}
			else
				shots[i] = 0;
		}

		time += 1.0f/60.0f;
	}

	return 0;
}
