#ifndef _WIN32
	#include <unistd.h>
	#include <arpa/inet.h>
	#define INVALID_SOCKET -1
	#define SOCKET_ERROR   -1
#else
	#include <winsock2.h>
	#include <windows.h>
	#define close(a) closesocket(a)
	#pragma comment(lib, "ws2_32.lib")
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "const.h"

#define bufferSize  (1 << 23)
int socketDescriptor;
int listen_socket;
int lenBuffer;
char sendBuff[bufferSize];

#include "LSocket.h"

JNIEXPORT void JNICALL Java_LSocket_connectServer
  (JNIEnv *penv, jobject jobj, jstring server, jint port){
#ifdef _WIN32
	WSADATA wsaData;
	int iResult = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (iResult != 0){
		fprintf(stderr, "WSAStartup failed: %d\n", iResult);
		return;
	}
#endif

	socketDescriptor = socket(AF_INET, SOCK_STREAM, 0);

	if (socketDescriptor == INVALID_SOCKET){
		fprintf(stderr, "EXIT_OPEN_SOCKET_FAILED\n");
		exit(EXIT_OPEN_SOCKET_FAILED);

	}

	struct sockaddr_in addr;
	jboolean isCopy;
	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	const char* str_value = penv->GetStringUTFChars(server, &isCopy);
	addr.sin_addr.s_addr = inet_addr(str_value);

	if (isCopy){
		penv->ReleaseStringUTFChars(server, str_value);
	}
	int error = connect(socketDescriptor, (struct sockaddr *)&addr, sizeof(addr));

	if (error == SOCKET_ERROR){
		fprintf(stderr, "EXIT_CONNECTION_FAILED\n");
		exit(EXIT_CONNECTION_FAILED);
	}
  }

void Receive(char* buff, int lenBuffer){
	int len = 0;
	while (len < lenBuffer){
		int currlen = recv(socketDescriptor, buff + len, lenBuffer - len, 0);
		if (currlen == SOCKET_ERROR){
			fprintf(stderr, "EXIT_CONNECTION_FAILED\n");
			exit(EXIT_CONNECTION_FAILED);
		}
		if (currlen == 0){
			fprintf(stderr, "Connection has been closed\n");
			exit(EXIT_SOCKET_CLOSED);
		}
		len += currlen;
	}
}

/*
 * Class:     LSocket
 * Method:    nextInt
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_LSocket_nextInt
  (JNIEnv *, jobject){
	char buff[8];
	Receive(buff, 4);
	return *(int *)buff;
  }

/*
 * Class:     LSocket
 * Method:    nextFloat
 * Signature: ()F
 */
JNIEXPORT jfloat JNICALL Java_LSocket_nextFloat
  (JNIEnv *, jobject){
	char buff[8];
	Receive(buff, 4);
	return *(float *)buff;
  }

/*
 * Class:     LSocket
 * Method:    writeInt
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_LSocket_writeInt
  (JNIEnv *, jobject, jint val){
	if (lenBuffer + 4 > bufferSize) return;
	*(int *)(sendBuff + lenBuffer) = val;
	lenBuffer += 4;
  }

/*
 * Class:     LSocket
 * Method:    writeFloat
 * Signature: (F)V
 */
JNIEXPORT void JNICALL Java_LSocket_writeFloat
  (JNIEnv *, jobject, jfloat val){
	if (lenBuffer + 4 > bufferSize) return;
	*(float *)(sendBuff + lenBuffer) = val;
	lenBuffer += 4;
  }

/*
 * Class:     LSocket
 * Method:    flush
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_LSocket_flush
  (JNIEnv *, jobject){
	int sendLen = 0;
	while (sendLen < lenBuffer){
		int sendRes = send(socketDescriptor, sendBuff + sendLen, lenBuffer - sendLen, 0);
		if (sendRes == SOCKET_ERROR){
			fprintf(stderr, "EXIT_CONNECTION_FAILED\n");
			exit(EXIT_CONNECTION_FAILED);
		}
		sendLen += sendRes;
	}
	lenBuffer = 0;
  }

/*
 * Class:     LSocket
 * Method:    closeSocket
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_LSocket_closeSocket
  (JNIEnv *, jobject){
	close(socketDescriptor);
#ifdef _WIN32
	WSACleanup();
#endif
  }
