import java.io.*;


class World{
	private static final int WORLD_BEGIN_MARKER = 1243635;
	private static final int WORLD_END_MARKER = 5365456;
	private static final int CLIENT_BEGIN_MARKER = 42345;
	private static final int CLIENT_END_MARKER = 8752845;

	public static final int MAXMONSTERS = 256;
	public static final int MAXMISSILES = 256;
	private LSocket socket;
	private java.util.concurrent.locks.ReentrantLock mutexLock;
	public CPlayer player;
	public CMonster[] monsters;
	public CMissile[] missiles;
	public World(String server, int port){
			socket = new LSocket();
			socket.connectServer(server, port);
			player = new CPlayer();
			monsters = new CMonster[MAXMONSTERS];
			for (int i = 0; i < MAXMONSTERS; i++)
				monsters[i] = new CMonster();
			missiles = new CMissile[MAXMISSILES];
			for (int i = 0; i < MAXMISSILES; i++)
				missiles[i] = new CMissile();
			mutexLock = new java.util.concurrent.locks.ReentrantLock();
	}
	public void makeMove(){
		try{
			long startTimeMs = System.currentTimeMillis( );
			mutexLock.lock();
			socket.writeInt(CLIENT_BEGIN_MARKER);
			player.write(socket);
			socket.writeInt(CLIENT_END_MARKER);
			socket.flush();
			startTimeMs = System.currentTimeMillis( );

			int marker = socket.nextInt();
			if (marker < WORLD_BEGIN_MARKER)
				throw new IOException();

			player.read(socket);
			if (player.HitPoints < 0.0){
				System.exit(0);
			}
			for (int i = 0; i < MAXMONSTERS; i++)
				monsters[i].read(socket);
			for (int i = 0; i < MAXMISSILES; i++)
				missiles[i].read(socket);

			marker = socket.nextInt();
			if (marker != WORLD_END_MARKER)
				throw new IOException();
			startTimeMs = System.currentTimeMillis( );
			mutexLock.unlock();
		}catch (IOException e){
			System.exit(1);
		}
	}
	public void run(float _vx, float _vy){
		player.RunX = _vx;
		player.RunY = _vy;
	}
	public void shoot(float _shootx, float _shooty){
		player.ShootX = _shootx;
		player.ShootY = _shooty;
	}
	public void lock(){
		mutexLock.lock();
	}
	public void unlock(){
		mutexLock.unlock();
	}
	protected void finalize(){
			socket.closeSocket();
	}
}
