package WSOnet;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

class BufferedDataInputStream {
    private byte[] buffer;
    private int[] int_buffer;
    private InputStream stream;
    private int l, r;
    private static final int buf_size = 1 << 16;
    private static final int int_buf_size = 1 << 14;
    
    public BufferedDataInputStream(InputStream in) {
        stream = in;
        buffer = new byte[buf_size];
        int_buffer = new int[int_buf_size];
        l = r = 0;
    }
    public int available() {
        if(l <= r) return r-l;
        return buffer.length - l + r;
    }
    public int readInt() throws IOException {
        if(l == int_buf_size) l = 0;
        if(l == r) refill();
        return int_buffer[l++];
    }

    void refill() throws IOException {
        int to_read = Math.min(buf_size - 4, Math.max(4, (stream.available() & 0xFFFFFFFC))), read = 0;
        while(read < to_read)
            read += stream.read(buffer, read, to_read-read);
        //System.err.format("Size: %d\n", read);
        for(int i = 0; i < to_read; i += 4) {
            int res = (buffer[i + 3] & 0xFF) << 24;
            res |= (buffer[i + 2] & 0xFF) << 16;
            res |= (buffer[i + 1] & 0xFF) << 8;
            res |= buffer[i] & 0xFF;
            int_buffer[r++] = res;
            if(r >= int_buf_size) r = 0;
        }
    }
    public void close() throws IOException {
        buffer = null;
        stream.close();
    }
}

public class Socket{
	private BufferedDataInputStream dis; 
	private DataOutputStream dos; 
	java.net.Socket s;
	public Socket(String server, int port) throws IOException{
		s = new java.net.Socket(server, port);
		dis = new BufferedDataInputStream(s.getInputStream());
		dos = new DataOutputStream(s.getOutputStream());
	}

	public void close() throws IOException{
		dis.close();
		dos.close();
		s.close();
	}

	public int nextInt() throws IOException{
		return dis.readInt();
	}

	public float nextFloat() throws IOException{
		return Float.intBitsToFloat( dis.readInt());
	}

	public void writeInt(int value) throws IOException{
		dos.writeInt(Integer.reverseBytes(value));
	}

	public void writeFloat(float value) throws IOException{
		int accum = Float.floatToIntBits(value);
		dos.writeInt(Integer.reverseBytes(accum));
	}
	public void flush() throws IOException{
		dos.flush();
	}
}
