#ifndef AI_HEADER_980387
#define AI_HEADER_980387

#include "Global.h"
#include "Weapons.h"
struct Entity;
struct Monster;

struct AIBase {
	typedef std::vector<Entity *> EntitySet;

	//the monster which has the current AI
	Monster *owner;
	//AI does not see non-player entities further that this distance
	double awarenessradius;

	inline AIBase(Monster *_owner, double _awarenessradius)
		 : owner(_owner), awarenessradius(_awarenessradius) {}
	inline virtual ~AIBase() {}

	inline virtual double GetAwarenessRadius() { return awarenessradius; }
	inline virtual void PassTime(double deltatime) {}
	inline virtual void Think(EntitySet &awareof) {}
};

struct RandomAI : public AIBase {
	//change velocity cooldown
	Cooldown cooldown;
	//move direction
	double2 dir;

	RandomAI(Monster *_owner, double _awarenessradius);
	void PassTime(double deltatime);
	void Think(EntitySet &awareof);
};

//DRONE
//tries to eat player as fast as he can
//also shoots if has gun
//anti-collision + slight random fluctuation
struct DroneAI : public AIBase {
	//cooldown for thinking
	Cooldown cooldown;
	//ai parameters
	const double playerimportance;
	const double randomimportance;

	DroneAI(Monster *_owner, double _awarenessradius
		, double _thinkperiod, double _playerimportance, double _randomimportance);
	void PassTime(double deltatime);
	void Think(EntitySet &awareof);
};

//DUCK
//does nothing
struct DuckAI : public AIBase {
	inline DuckAI(Monster *_owner, double _awarenessradius) : AIBase(_owner, _awarenessradius) {}
};

//WALL
//maintains constant speed
struct WallAI : public AIBase {
	//move direction
	double2 direction;
	//side direction
	double2 side;
	//the position he must be in now
	double2 desiredposition;

	WallAI(Monster *_owner, double _awarenessradius, double2 _direction);
	void PassTime(double deltatime);
	void Think(EntitySet &awareof);
};

//DODGER
//same as drone, but tries to dodge missiles
struct DodgerAI : public AIBase {
	//cooldown for thinking
	Cooldown cooldown;
	//ai parameters
	const double playerimportance;
	const double randomimportance;
	const double missileimportance;

	DodgerAI(Monster *_owner, double _awarenessradius
		, double _thinkperiod, double _playerimportance, double _randomimportance, double _missileimportance);
	void PassTime(double deltatime);
	void Think(EntitySet &awareof);
};

#endif
