#include "socket.h"
#include "world.h"
#include "socket.h"
#include "simpleclass.h"
#include <cassert>

#ifdef _WIN32
#include <windows.h>
#endif

struct WorldBase {
	Socket socket;
#ifdef _WIN32
	HANDLE ghMutex;
#endif
	WorldBase(const char *ip , int port) : socket(ip, port) {}
	WorldBase(int port) : socket(port) {}
};


World::World(){
}

void World::makeMove(){
	lock();
	base->socket.writeInt(CLIENT_BEGIN_MARKER);
	player.write(base->socket);
	base->socket.writeInt(CLIENT_END_MARKER);
	base->socket.flush();

	int marker = base->socket.nextInt();
	if (marker != WORLD_BEGIN_MARKER){
		fprintf(stderr, "Protocol error. Wrong WORLD_BEGIN_MARKER. Expected %d, got %d\n", WORLD_BEGIN_MARKER, marker);
		exit(EXIT_PROTOCOL_ERROR);
	}

	player.read(base->socket);
	for (int i = 0; i < MAXMONSTERS; i++)
		monsters[i].read(base->socket);

	for (int i = 0; i < MAXMISSILES; i++)
		missiles[i].read(base->socket);

	marker = base->socket.nextInt();
	if (marker != WORLD_END_MARKER){
		fprintf(stderr, "Protocol error. Wrong WORLD_END_MARKER. Expected %d, got %d\n", WORLD_END_MARKER, marker);
		exit(EXIT_PROTOCOL_ERROR);
	}

	unlock();
}
void World::getMove(){
	int marker = base->socket.nextInt();
//	fprintf(stderr, "begin marker : %d\n",marker);
	if (marker != CLIENT_BEGIN_MARKER){
		fprintf(stderr, "Protocol error. Wrong CLIENT_BEGIN_MARKER. Expected %d, got %d\n", CLIENT_BEGIN_MARKER, marker);
		exit(EXIT_PROTOCOL_ERROR);
	}

	player.read(base->socket);

	marker = base->socket.nextInt();

//	fprintf(stderr, "end marker : %d\n",marker);
	if (marker != CLIENT_END_MARKER){
		fprintf(stderr, "Protocol error. Wrong CLIENT_END_MARKER. Expected %d, got %d\n", CLIENT_END_MARKER, marker);
		exit(EXIT_PROTOCOL_ERROR);
	}
}

void World::sendWorld(){
	base->socket.writeInt(WORLD_BEGIN_MARKER);

	player.write(base->socket);
	for (int i = 0; i < MAXMONSTERS; i++)
		monsters[i].write(base->socket);

	for (int i = 0; i < MAXMISSILES; i++)
		missiles[i].write(base->socket);

	base->socket.writeInt(WORLD_END_MARKER);
	base->socket.flush();
}

void World::run(float _vx, float _vy){
	player.RunX = _vx;
	player.RunY = _vy;
}

void World::shoot(float _shootx, float _shooty){
	player.ShootX = _shootx;
	player.ShootY = _shooty;
}

void World::lock(){
#ifdef _WIN32
	 WaitForSingleObject(base->ghMutex, INFINITE); 
#endif
}

void World::unlock(){
#ifdef _WIN32
	ReleaseMutex(base->ghMutex);
#endif
}

World::~World(){
#ifdef _WIN32
	CloseHandle(base->ghMutex);
#endif
}

bool World::readFrameFromFile(FILE *f){
	if (fread(&player, 1, sizeof(CPlayer), f) != sizeof(CPlayer)) return false;
	if (fread(&monsters, 1, sizeof(monsters), f) != sizeof(monsters)) return false;
	if (fread(&missiles, 1, sizeof(missiles), f) != sizeof(missiles)) return false;
	return true;
}

bool World::writeFrameToFile(FILE *f){
	assert(fwrite(&player, 1, sizeof(CPlayer), f) == sizeof(CPlayer));
	assert(fwrite(&monsters, 1, sizeof(monsters), f) == sizeof(monsters));
	assert(fwrite(&missiles, 1, sizeof(missiles), f) == sizeof(missiles));
	return true;
}
